from enum import Enum, IntEnum

from colored import fg, stylize

from .environment import EnvironmentVariable


class Color(Enum):
    ERROR = fg("red")
    NOTICE = fg("yellow")


class MessageLevel(IntEnum):
    LOUD = 2
    NORMAL = 1
    QUIET = 0


def _level() -> MessageLevel:
    """Minium threshold level for logging to occur"""
    from .environment import is_verbose_logging

    return MessageLevel.QUIET if is_verbose_logging() else MessageLevel.NORMAL


def _print(*args, level=MessageLevel.NORMAL):
    if level < _level():
        return

    print(*args)


def _stylize(text: str, color: Color) -> str:
    return stylize(text, color.value)


def _print_stylized(text: str, color: Color, level=MessageLevel.NORMAL):
    _print(_stylize(text, color), level=level)


def _print_error(text: str):
    _print_stylized(text, Color.ERROR, MessageLevel.LOUD)


def print_env_var_missing(env_var: EnvironmentVariable):
    _print_error(f"Environment variable '{env_var.value}' missing")


def print_using_version(version: str, bin_path: str, nvmrc_path: str = None):
    messages = (
        f"Found '{nvmrc_path}' with version <{version}>"
        if nvmrc_path
        else f"Using <{version}> version",
        f"\n.{bin_path}\n",
    )
    _print("".join(messages), level=MessageLevel.QUIET)


def print_node_bin_file_does_not_exist(bin_path: str):
    _print(f"No executable file found at '{bin_path}'", level=MessageLevel.LOUD)


def print_version_not_installed(version: str):
    _print_error(f"N/A version '{version}' is not yet installed.\n")
    _print(
        "You need to run",
        _stylize(f"'nvm install {version}'", Color.NOTICE),
        "to install it before using it.\n",
    )
    _print(
        "Or set the environment variable",
        _stylize(f"'{EnvironmentVariable.AUTO_INSTALL.value}'", Color.NOTICE),
        "to auto install at run time.\n",
    )


def print_running_version(version_number: str):
    _print(f"Executing shim version {version_number}", level=MessageLevel.QUIET)


def print_unable_to_get_stable_version():
    _print_error("Unable to retrieve stable version from nvm")


def print_process_interrupted(exc: KeyboardInterrupt):
    _print(f"\nInterrupted. {exc}")



import runpy
import sys

import pytest

from nvshim.core.shim import main
from nvshim.utils.constants import ErrorCode
from nvshim.utils.process import clean_output


@pytest.fixture
def test_shim_args():
    initial_args = list(sys.argv)
    sys.argv = ["/full/path/to/shim/node", "--version", "--help"]
    yield sys.argv
    sys.argv = initial_args


def test_shim_executes_with_args(mocker, test_shim_args):
    mocked_core_main = mocker.patch("nvshim.core.shim.core.main", autospec=True)
    main()
    mocked_core_main.assert_called_once_with()


def test_shim_handles_process_interrupt(mocker, capsys, test_shim_args, snapshot):
    mocked_core_main = mocker.patch(
        "nvshim.core.shim.core.main",
        autospec=True,
        side_effect=KeyboardInterrupt("Ctrl+C"),
    )
    mocked_sys_exit = mocker.patch("sys.exit")
    main()
    mocked_core_main.assert_called_once_with()
    mocked_sys_exit.assert_called_once_with(ErrorCode.KEYBOARD_INTERRUPT)
    captured = capsys.readouterr()
    snapshot.assert_match(clean_output(captured.out))



from contextlib import contextmanager
from enum import Enum
import json
import os
from typing import Dict

EnvDict = Dict[str, str]


class EnvironmentVariable(Enum):
    AUTO_INSTALL = "NVSHIM_AUTO_INSTALL"
    NVM_DIR = "NVM_DIR"
    VERBOSE = "NVSHIM_VERBOSE"


class MissingEnvironmentVariableError(Exception):
    def __init__(self, env_var: EnvironmentVariable):
        self.env_var = env_var


def _get_env_var(env_var: EnvironmentVariable, raise_missing: bool = False) -> object:
    try:
        return json.loads(os.environ[env_var.value])
    except KeyError:
        if raise_missing:
            raise MissingEnvironmentVariableError(env_var)
    except json.decoder.JSONDecodeError:
        return os.environ.get(env_var.value)


def _set_envs(values: EnvDict):
    os.environ.clear()
    os.environ.update(**values)


@contextmanager
def process_env(env_vars: EnvDict):
    prev_env_vars = dict(os.environ)
    _set_envs(env_vars)
    yield
    _set_envs(prev_env_vars)


def is_version_auto_install_enabled() -> bool:
    return bool(_get_env_var(EnvironmentVariable.AUTO_INSTALL))


def is_verbose_logging() -> bool:
    return _get_env_var(EnvironmentVariable.VERBOSE)


def get_nvm_dir() -> str:
    return _get_env_var(EnvironmentVariable.NVM_DIR, True)



import argparse
import functools
import os
import re
import subprocess
import sys
from typing import Callable, Dict, List, Sequence, Set, Union

import semver

from nvshim import __version__
from nvshim.utils import environment, message, process
from nvshim.utils.constants import ErrorCode


AliasResolver = Callable[..., str]
AliasOrResolver = Union[str, AliasResolver]
AliasMapping = Dict[str, AliasOrResolver]
VersionMapping = Dict[str, str]


class HashableList(list):
    def __hash__(self):
        return hash(frozenset(self))


class HashableSet(set):
    def __hash__(self):
        return hash(frozenset(self))


class HashableDict(dict):
    def __hash__(self):
        return hash(frozenset(self.items()))


def get_files(path: str) -> [str]:
    """
    Generate the file paths to traverse, or a single path if a file name was given
    """
    if os.path.isdir(path):
        for root, _, files in os.walk(path):
            for name in files:
                yield os.path.join(root, name)
    elif os.path.exists(path):
        yield path


def _run_nvm_cmd(
    nvm_sh_path: str, nvm_args: str, **kwargs: dict
) -> process.subprocess.CompletedProcess:
    return process.run(
        f". {nvm_sh_path} && nvm {nvm_args}", shell="bash", encoding="UTF-8", **kwargs
    )


@functools.lru_cache(maxsize=None)
def get_nvm_stable_version(nvm_dir) -> str:
    output = _run_nvm_cmd(
        get_nvmsh_path(nvm_dir), "alias stable", stdout=subprocess.PIPE
    ).stdout
    try:
        return re.findall(r"> v([\w\.]+)", process.clean_output(output))[0]
    except (IndexError, KeyError):
        message.print_unable_to_get_stable_version()


def get_nvm_aliases_dir(nvm_dir: str) -> str:
    return os.path.join(nvm_dir, "alias")


def get_nvm_aliases(nvm_dir: str) -> AliasMapping:
    aliases_to_version = HashableDict(
        default="stable", node="stable", stable=lambda: get_nvm_stable_version(nvm_dir)
    )
    nvm_aliases_dir = get_nvm_aliases_dir(nvm_dir)
    for file_path in get_files(nvm_aliases_dir):
        rel_path = os.path.relpath(file_path, nvm_aliases_dir)
        with open(file_path) as f:
            aliases_to_version[rel_path] = f.readline().strip()

    return aliases_to_version


def parse_version(version: str) -> semver.VersionInfo:
    try:
        return semver.parse_version_info(
            version[1:] if version.startswith("v") else version
        )
    except ValueError:
        pass


@functools.lru_cache(maxsize=None)
def resolve_alias(
    var: AliasOrResolver,
    alias_mapping: AliasMapping,
    seen: Set[str] = HashableSet(),
    seen_order: List[str] = HashableList(),
) -> (semver.VersionInfo, str, [str]):
    """
    Resolve an alias to a semantic version going through multiple mappings

    :param var: version alias or resolver
    :type alias_mapping: mapping of alias to version, alias or resolver
    :return: (semantic version info, alias name, list of aliases traversed)
    """
    alias = var() if callable(var) else var
    if alias in alias_mapping and alias not in seen:
        seen_order.append(alias)
        seen.add(alias)
        return resolve_alias(alias_mapping.get(alias), alias_mapping, seen, seen_order)

    return parse_version(alias), alias, seen_order


@functools.lru_cache(maxsize=None)
def resolve_nvm_aliases(nvm_aliases: AliasMapping) -> AliasMapping:
    """
    Resolve all aliases in a mapping to their version info
    """
    resolved_aliases = {}
    for alias, version in nvm_aliases.items():
        version_info = (
            type(version) is str and parse_version(version)
        ) or resolve_alias(version, nvm_aliases)[0]

        if version_info:
            resolved_aliases[alias] = str(version_info)

    return resolved_aliases


def get_node_versions_dir(nvm_dir: str) -> str:
    return os.path.join(nvm_dir, "versions", "node")


def get_node_version_bin_dir(node_versions_dir: str, version: str) -> str:
    return os.path.join(node_versions_dir, f"v{version}", "bin")


def get_node_versions(node_versions_dir: str) -> VersionMapping:
    files = os.listdir(node_versions_dir) if os.path.exists(node_versions_dir) else []
    return {
        str(v): get_node_version_bin_dir(node_versions_dir, str(v))
        for v in map(parse_version, files)
    }


def merge_nvm_aliases_with_node_versions(
    nvm_aliases: AliasMapping, node_versions: VersionMapping
) -> VersionMapping:
    alias_versions = {
        alias: node_versions[version]
        for alias, version in nvm_aliases.items()
        if version in node_versions
    }
    return dict(alias_versions, **node_versions)


def get_nvmrc_path(exec_dir: str) -> str:
    root_dir = os.path.abspath(os.sep)
    current_dir = exec_dir
    config_file = ".nvmrc"
    config_found = False
    while True:
        current_config = os.path.join(current_dir, config_file)
        config_found = os.path.exists(current_config)
        if config_found or current_dir == root_dir:
            break
        current_dir = os.path.realpath(os.path.join(current_dir, "../"))

    return current_config if config_found else None


def get_nvmrc(nvmrc_path: str = None) -> (str, bool):
    version = "default"
    if nvmrc_path:
        with open(nvmrc_path) as f:
            rc_version = f.readline().strip()
            parsed_version = parse_version(rc_version)
            if parsed_version:
                return str(parsed_version), True
            version = rc_version

    return version, False


def get_nvmsh_path(nvm_dir: str) -> str:
    return os.path.join(nvm_dir, "nvm.sh")


def _pretty_version(version: str, parsed: bool) -> str:
    return f"v{version}" if parsed else version


def get_bin_path(
    *,
    version: str,
    is_version_parsed: bool,
    nvm_aliases: AliasMapping,
    node_versions: VersionMapping,
    bin_file: str,
    node_versions_dir: str,
    nvm_sh_path: str,
):
    if version not in node_versions and nvm_aliases.get(version) not in node_versions:
        nvm_aliases = resolve_nvm_aliases(nvm_aliases)
    version_mapping = merge_nvm_aliases_with_node_versions(nvm_aliases, node_versions)
    try:
        node_path = version_mapping[version]
    except KeyError:
        if not environment.is_version_auto_install_enabled():
            message.print_version_not_installed(
                _pretty_version(version, is_version_parsed)
            )
            sys.exit(ErrorCode.VERSION_NOT_INSTALLED)

        _run_nvm_cmd(nvm_sh_path, f"install {version}")
        node_path = get_node_version_bin_dir(
            node_versions_dir, version=nvm_aliases.get(version, version)
        )

    bin_path = os.path.join(node_path, bin_file)
    if not os.path.exists(bin_path):
        message.print_node_bin_file_does_not_exist(bin_path)
        sys.exit(ErrorCode.EXECUTABLE_NOT_FOUND)

    return bin_path


def parse_args(args: Sequence[str]) -> (argparse.Namespace, List[str]):
    """
    Get the arguments to be used to execute the node binary
    :return: parsed and unknown arguments
    """
    parser = argparse.ArgumentParser(
        description="Launch executable using project or default node version",
        add_help=False,
    )
    parser.add_argument(
        "bin_file",
        metavar="nvx",
        type=str,
        help="filename of the node executable binary",
    )
    parser.add_argument(
        "bin_args",
        metavar="args",
        nargs="*",
        help="arguments passed to the node executable",
    )

    return parser.parse_known_args(args)


def main(version_number: str = __version__):
    message.print_running_version(version_number)
    parsed_args, unknown_args = parse_args(sys.argv[1:])
    nvmrc_path = get_nvmrc_path(os.getcwd())
    version, parsed = get_nvmrc(nvmrc_path)
    try:
        nvm_dir = environment.get_nvm_dir()
    except environment.MissingEnvironmentVariableError as error:
        message.print_env_var_missing(error.env_var)
        sys.exit(ErrorCode.ENV_NVM_DIR_MISSING)

    bin_path = get_bin_path(
        version=version,
        is_version_parsed=parsed,
        nvm_aliases=get_nvm_aliases(nvm_dir),
        node_versions=get_node_versions(get_node_versions_dir(nvm_dir)),
        node_versions_dir=get_node_versions_dir(nvm_dir),
        bin_file=parsed_args.bin_file,
        nvm_sh_path=get_nvmsh_path(nvm_dir),
    )
    message.print_using_version(_pretty_version(version, parsed), bin_path, nvmrc_path)
    process.run(bin_path, *parsed_args.bin_args, *unknown_args)


if __name__ == "__main__":
    main()



import os
import sys
import shutil
import functools
from typing import Dict, Union

import pytest


__TEST_DIR__ = os.path.join(os.environ["PWD"], "tmp")
__TEST_DIR_STRUCTURE__ = {".nvmrc": "v8.16.2"}
__NVM_DIR__ = os.path.join(os.path.expanduser("~"), ".nvm")


@pytest.fixture
def test_args():
    initial_args = list(sys.argv)
    sys.argv = sys.argv[:1] + ["npm", "--version", "--help"]
    yield sys.argv
    sys.argv = initial_args


def _flatten_fs(base: str, structure: Dict[str, Union[str, dict]]):
    flattened = {}
    items = list(structure.items())
    while items:
        path, content = items.pop()
        file_path = os.path.realpath(os.path.join(base, path))
        content_type = type(content)
        if content_type is str:
            flattened[file_path] = content
        elif content_type is dict:
            flattened[file_path] = {}
            items.extend(
                (os.path.join(path, _path), _content)
                for _path, _content in content.items()
            )

    return flattened


def _make_fs(base: str, structure: Dict[str, str] = {}):
    os.makedirs(base, exist_ok=True)
    for file_path, content in _flatten_fs(base, structure).items():
        content_type = type(content)
        if content_type is str:
            with open(file_path, "w") as f:
                f.write(content)
        elif content_type is dict:
            os.makedirs(file_path, exist_ok=True)


@pytest.fixture
def test_workspace():
    _make_fs(__TEST_DIR__)
    yield __TEST_DIR__
    shutil.rmtree(__TEST_DIR__)


@pytest.fixture
def test_node_version_dir():
    nvm_dir = os.environ.get("NVM_DIR") or __NVM_DIR__
    version_path = os.path.join(nvm_dir, "versions", "node", "v8.16.2")
    shutil.rmtree(version_path, ignore_errors=True)
    return __NVM_DIR__, version_path


@pytest.fixture
def test_workspace_with_nvmrc(test_workspace: str):
    _make_fs(test_workspace, __TEST_DIR_STRUCTURE__)
    yield test_workspace


@pytest.fixture
def test_nested_workspace_with_nvmrc(test_workspace: str):
    nested_workspace = os.path.join(test_workspace, "nest", "project")
    _make_fs(test_workspace, {nested_workspace: {}, **__TEST_DIR_STRUCTURE__})
    yield nested_workspace



import os
import sys

import nvshim.core.__main__ as core
from nvshim.utils.constants import ErrorCode
from nvshim.utils.message import print_process_interrupted


def main():
    try:
        sys.argv.insert(1, os.path.basename(sys.argv[0]))
        core.main()
    except KeyboardInterrupt as e:
        print_process_interrupted(e)
        sys.exit(ErrorCode.KEYBOARD_INTERRUPT)


if __name__ == "__main__":
    main()



import pytest
import os
import runpy
import shutil
import subprocess

from nvshim.utils.constants import ErrorCode
from nvshim.utils.environment import process_env, EnvironmentVariable
from nvshim.utils.process import clean_output
from nvshim.core.__main__ import (
    get_node_version_bin_dir,
    get_node_versions_dir,
    main,
    parse_args,
)


class TestParseArgs:
    def test_raises_missing_bin_file(self):
        with pytest.raises(SystemExit):
            parse_args([])

    def test_accepts_bin_file_arg(self, snapshot):
        snapshot.assert_match(parse_args(["node"]))

    def test_accepts_any_arg_for_bin_file(self, snapshot):
        snapshot.assert_match(parse_args(["npm", "--version", "--help"]))


class TestMain:
    def test_fails_when_nvm_dir_not_available(
        self, mocker, capsys, snapshot, test_args
    ):
        with process_env({EnvironmentVariable.VERBOSE.value: "true"}), pytest.raises(
            SystemExit
        ) as exc_info:
            main()

        assert exc_info.value.code == 1003
        captured = capsys.readouterr()
        snapshot.assert_match(clean_output(captured.out))
        assert not captured.err

    def test_fails_when_version_not_installed(
        self,
        mocker,
        capsys,
        snapshot,
        test_args,
        test_nested_workspace_with_nvmrc,
        test_node_version_dir,
    ):
        nvm_dir, _ = test_node_version_dir
        mocker.patch(
            "nvshim.core.__main__.os.getcwd",
            autospec=True,
            return_value=test_nested_workspace_with_nvmrc,
        )
        mock_env = {
            EnvironmentVariable.NVM_DIR.value: nvm_dir,
            **os.environ,
            EnvironmentVariable.VERBOSE.value: "true",
            EnvironmentVariable.AUTO_INSTALL.value: "false",
        }
        with process_env(mock_env), pytest.raises(SystemExit) as exc_info:
            main()

        assert exc_info.value.code == 1001
        captured = capsys.readouterr()
        snapshot.assert_match(clean_output(captured.out))
        assert not captured.err

    def test_runs_correct_version_of_node(
        self,
        mocker,
        capsys,
        snapshot,
        test_args,
        test_nested_workspace_with_nvmrc,
        test_node_version_dir,
    ):
        nvm_dir, node_version_dir = test_node_version_dir
        mocker.patch(
            "nvshim.core.__main__.os.getcwd",
            autospec=True,
            return_value=test_nested_workspace_with_nvmrc,
        )
        mocked_process_run = mocker.patch(
            "nvshim.utils.process.subprocess.run", wraps=subprocess.run
        )
        mock_env = {
            EnvironmentVariable.NVM_DIR.value: nvm_dir,
            **os.environ,
            EnvironmentVariable.VERBOSE.value: "true",
            EnvironmentVariable.AUTO_INSTALL.value: "true",
        }
        with process_env(mock_env):
            main()

        mocked_process_run.assert_called_with(
            (f"{node_version_dir}/bin/{test_args[1]}", *test_args[2:]), check=True
        )
        captured = capsys.readouterr()
        assert "with version <v8.16.2>" in clean_output(captured.out)
        assert not captured.err



import os
import re
import subprocess
import sys
from typing import Dict

from .environment import EnvironmentVariable, EnvDict, process_env


def _with_venv(env: EnvDict):
    path_key = "PATH"
    env_path = env.get(path_key, "")
    return dict(env, **{path_key: f"venv/bin/:{env_path}"})


def run(*args, **kwargs) -> subprocess.CompletedProcess:
    env_vars = _with_venv(os.environ)
    env_vars[EnvironmentVariable.AUTO_INSTALL.value] = "false"

    try:
        with process_env(env_vars):
            return subprocess.run(args, **kwargs, check=True)
    except subprocess.CalledProcessError as error:
        sys.exit(error.returncode)


def clean_output(output: str) -> str:
    """Removes ansi color codes from string"""
    return re.sub(r"\x1B[@-_][0-?]*[ -/]*[@-~]", "", str(output).strip())



import subprocess
import pytest

from nvshim.utils import process


def test_process_run_completes_successfully():
    output = process.run("echo", "success", stdout=subprocess.PIPE).stdout.strip()
    assert output == b"success"


def test_process_run_raises_correct_exception():
    with pytest.raises(SystemExit) as exc_info:
        process.run("bash", "-c", "exit 1")

    assert exc_info.value.code == 1



from enum import IntEnum


class ErrorCode(IntEnum):
    ENV_NVM_DIR_MISSING = 1003
    EXECUTABLE_NOT_FOUND = 1002
    KEYBOARD_INTERRUPT = 130
    VERSION_NOT_INSTALLED = 1001


shims = {"node", "npm", "npx"}



import sys
from datetime import datetime

sys.path.append("src")

import os
from setuptools import setup
from typing import Sequence

from nvshim.utils.constants import shims


def readme() -> str:
    with open("README.md") as f:
        return f.read()


def lines(filepath) -> Sequence[str]:
    """Lines of a file generator"""
    with open(filepath) as f:
        while True:
            line = f.readline().strip()
            if line:
                yield line
            else:
                return


def get_requirements(filepath: str, visited: [str]) -> [str]:
    """
    Get all pip requirements specified by a requirements file
    with support for nested requirements files

    :param filepath: path to requirements.txt file
    :param visited: mutable list of visited requirements.txt files
    :return: unordered list of requirements without versions
    """
    requirements = set()
    filepath = os.path.realpath(filepath)
    rel_filepath = os.path.relpath(filepath)

    if filepath in visited:
        print("Skipping requirements:", rel_filepath)
        print(visited)

    else:
        print("Parsing requirements:", rel_filepath)
        visited.append(filepath)
        req_file_delim = "-r"
        req_package_delim = "=="
        requirements_dir = os.path.dirname(filepath)
        for line in lines(filepath):
            if line.startswith("#"):
                continue
            if line.startswith(req_file_delim):
                nested_req_file = line.split(req_file_delim)[1].strip()
                nested_req_filepath = os.path.join(requirements_dir, nested_req_file)
                requirements.union(get_requirements(nested_req_filepath, visited))
            elif req_package_delim in line:
                requirements.add(line.split(req_package_delim)[0])

    return list(requirements)


def version_scheme(version):
    if version.exact:
        return version.format_with("{tag}")
    else:
        return datetime.now().strftime("%Y.%m.%d.%H%M%S%f")


setup(
    author="Emmanuel Ogbizi-Ugbe",
    author_email="iamogbz+pypi@gmail.com",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Topic :: Software Development",
    ],
    description="Automagically use the correct version of node",
    entry_points={"console_scripts": [f"{s}=nvshim.core.shim:main" for s in shims]},
    include_package_data=True,
    install_requires=get_requirements("requirements/prod.txt", []),
    keywords="node nvm node-shim shim shell nvm-shim",
    long_description=readme(),
    long_description_content_type="text/markdown",
    license="GNU",
    name="nvshim",
    packages=["nvshim", "nvshim.core", "nvshim.utils"],
    package_dir={"": "src"},
    setup_requires=["setuptools_scm"],
    tests_require=get_requirements("requirements/test.txt", []),
    url="http://github.com/iamogbz/nvshim",
    use_scm_version={
        "local_scheme": lambda _: "",
        "version_scheme": version_scheme,
        "write_to": "./src/nvshim/__init__.py",
        "write_to_template": '__version__ = "{version}"\n',
    },
)



import os
import importlib

import semver

import nvshim
from nvshim.utils import process


dist_path = "dist"


def _is_valid_release_version(version: str) -> bool:
    try:
        return semver.parse_version_info(version)
    except:
        pass


def _clean():
    print("Cleaning...")
    process.run("rm", "-rf", dist_path)


def _build():
    print("Building...")
    process.run("python", "setup.py", "sdist", "bdist_wheel")


def _get_publish_command(*, dry_run: bool = True):
    cmd = ["twine", "upload", "--skip-existing"]
    if dry_run:
        cmd.extend(["--repository-url", "https://test.pypi.org/legacy/"])
    cmd.append(os.path.join(dist_path, "*"))
    return cmd


def _publish(*, dry_run: bool = True):
    print("Target:", "test.pypi.org" if dry_run else "pypi.org")
    process.run(*_get_publish_command(dry_run=dry_run))

    print("Publishing completed.")


def main():
    _clean()
    _build()

    __version__ = importlib.reload(nvshim).__version__
    print(f"Publishing: {__version__}")

    if __version__:
        _publish(dry_run=True)
    else:
        return print("No build version")

    if _is_valid_release_version(__version__):
        _publish(dry_run=False)
    else:
        return print("No release version")


if __name__ == "__main__":
    main()
